@@ -13,9 +13,10 @@ import pdftitle
 import re
 import logging
 from googlesearch import search
-from . import bibtex_makers
 import pdf2doi.config as config
 import os
+import feedparser
+
 logger = logging.getLogger('pdf2doi')
 
 ######## Beginning first part, low-level functions ######## 
@@ -53,12 +54,67 @@ arxiv_regexp = ['arxiv[\s]*\:[\s]*(\d{4}\.\d+)(?:v\d+)?(?:[\s\n\"<]|$)',  #versi
                                                                             #This is helpful when we are trying to extrat the arXiv ID from the file name.
                 '\A(\d{4}\.\d+)(?:v\d+)?$']                               #version 2 is similar to version 0, without the requirement of "arxiv : " at the beginning 
                                                                             #but requires that the string contains ONLY the arXiv ID.
-    
+
+def validate_doi_web(doi,method=None):
+    """
+    It queries dx.doi.org for a certain doi, to check that the doi exists.
+    If dx.doi.org could not find any paper associated to this doi, the function returns None
+    If it was not possible to connect to dx.doi.org, the function returns -1
+    If dx.doi.org confirmed that DOI exists, the function returns the full text obtained from dx.doi.org
+    Depending on the value of method (default =config.get('method_dxdoiorg')), the format of the text returned by dx.doi.org will be different
+    """
+    if method == None:
+        method = config.get('method_dxdoiorg')
+    try:
+        url = "http://dx.doi.org/" + doi
+        headers = {"accept": method}
+        NumberAttempts = 10
+        while NumberAttempts:
+            r = requests.get(url, headers = headers)
+            r.encoding = 'utf-8' #This forces to encode the obtained text with utf-8
+            text = r.text
+            if (text.lower().find("503 Service Unavailable".lower() )>=0) or (not text):
+                NumberAttempts = NumberAttempts -1
+                logging.info("Could not reach dx.doi.org. Trying again. Attempts left: " + str(NumberAttempts))
+                continue
+            else:
+                NumberAttempts = 0
+            if (text.lower().find( "DOI Not Found".lower() ))==-1:
+                return text
+                #metadata = parse_bib_from_dxdoiorg(text, method)
+                #return {'bibtex_entry':make_bibtex(metadata), 'bibtex_data':metadata}
+            else:
+                return None
+    except Exception as e:
+        logger.error(r"Some error occured within the function validate_doi_web")
+        logger.error(e)
+        return -1
+
+def validate_arxivID_web(arxivID):
+    """
+    It queries export.arxiv.org for a certain arxiv ID, to check that it exists.
+    If export.arxiv.org could not find any paper associated to this arxiv ID, the function returns None
+    If it was not possible to connect to export.arxiv.org, the function returns -1
+    If export.arxiv.org confirmed that DOI exists, the function returns the data obtained from export.arxiv.org
+    """
+    try:
+        url = "http://export.arxiv.org/api/query?search_query=id:" + arxivID
+        result = feedparser.parse(url)
+        items = result.entries[0]
+        found = len(items) > 0
+        if not found: 
+            return None
+        else:
+            return items
+    except Exception as e:
+        logger.error(r"Some error occured within the function arxiv2bib")
+        logger.error(e)
+        return -1    
 
 def validate(identifier,what='doi'):
     """
     Check that an identifier is a valid identifier by using a regular expression.
-    If config.check_online_to_validate, it also checks that the identifier is actually associate to a paper via 
+    If config.get('webvalidation') == true, it also checks that the identifier is actually associate to a paper via 
     a query to the proper website (e.g. http://dx.doi.org/ for DOIs).
 
     Parameters
@@ -69,50 +125,50 @@ def validate(identifier,what='doi'):
         Specifies the kind of identifier to be validated.
     Returns
     -------
-    result : It is either boolean (True if identifier is valid, False if is not valid) or a string
-        If config.check_online_to_validate is set to true, the validation process also generate a valid bibtex entry, which is 
-        returned a string in result
+    result :    If config.get('webvalidation') is set to true but connection was not possible, it returns None
+                If config.get('webvalidation') is set to true and the identifier is validated, it returns the validation object (could be either a string or dictionary)
+                If config.get('webvalidation') is set to false, it returns a single boolean value (True if identifier is valid, False if is not valid)
     """  
     if not identifier:
         return None
     if what=='doi':
         if re.match(doi_pattern,identifier,re.I):
-            if config.check_online_to_validate:
+            if config.get('webvalidation'):
                 logger.info(f"Validating the possible DOI {identifier} via a query to dx.doi.org...")
-                result = bibtex_makers.doi2bib(identifier)
+                result = validate_doi_web(identifier)
                 if result==-1:
                     logger.error(f"Some error occured during connection to dx.doi.org.")
                     return None
                 if isinstance(result,str) and result.strip()[0:5] == '@misc':
-                    logger.error(f"A valid bibTex entry was returned by dx.doi.org, but it starts with the tag \"@misc\". This might be the DOI of the journal and not the article itself.")
-                    return None
+                    logger.error(f"The DOI was validated by by dx.doi.org, but the validation string starts with the tag \"@misc\". This might be the DOI of the journal and not the article itself.")
+                    return False
                 if result:
-                    logger.info(f"The DOI {identifier} is validated by dx.doi.org. A bibtex entry was also created.")
+                    logger.info(f"The DOI {identifier} is validated by dx.doi.org.")
                     return result
                 else:
                     logger.info(f"The DOI {identifier} is not valid according to dx.doi.org.")
                     return False
             else:
-                logger.info(f"(web validation is deactivated. Set webvalidation = True in order to validate a potential DOI on dx.doi.org).")
+                logger.info(f"NOTE: Web validation is deactivated. Set webvalidation = True (or remove the '-nwv' argument if working from command line) in order to validate a potential DOI on dx.doi.org.")
                 return True
         else: return False
 
     elif what=='arxiv':
         if re.match(arxiv2007_pattern,identifier,re.I):
-            if config.check_online_to_validate:
+            if config.get('webvalidation'):
                 logger.info(f"Validating the possible arxiv ID {identifier} via a query to export.arxiv.org...")
-                result = bibtex_makers.arxiv2bib(identifier)
+                result = validate_arxivID_web(identifier)
                 if result==-1:
                     logger.error(f"Some error occured during connection to export.arxiv.org.")
                     return None
                 if result:
-                    logger.info(f"The Arxiv ID {identifier} is validated by export.arxiv.org. A bibtex entry was also created.")
+                    logger.info(f"The Arxiv ID {identifier} is validated by export.arxiv.org")
                     return result
                 else:
                     logger.info(f"The Arxiv ID {identifier} is not valid according to export.arxiv.org.")
                     return False
             else:
-                logging.warning(f"(web validation is deactivated. Set webvalidation = True in order to validate a potential arxiv ID on export.arxiv.org).")
+                logger.info(f"NOTE: Web validation is deactivated. Set webvalidation = True (or remove the '-nwv' argument if working from command line) in order to validate a potential arxiv ID on export.arxiv.org.")
                 return True
         else: return False
 
@@ -135,12 +191,11 @@ def extract_arxivID_from_text(text,version=0):
     Returns
     -------
     arxivID : string or None
-        It returns the arxiv ID if any was found, or None
+        It returns a list of all arxiv IDs found
     """   
     try:                                            
-        regexDOI = re.search(arxiv_regexp[version] ,text,re.I)
-        if regexDOI:
-            return regexDOI.group(1)
+        arxiv_ids = re.findall(arxiv_regexp[version] ,text,re.I)
+        return arxiv_ids
     except:
         pass
     return None
@@ -160,19 +215,18 @@ def extract_doi_from_text(text,version=0):
 
     Returns
     -------
-    doi : string or None
-        It returns the doi if any was found, or None
+    doi_list : string or None
+        It returns a list of all dois found
 
     """    
     try:
-        regexDOI = re.search(doi_regexp[version],text,re.I)
-        if regexDOI:
-            return regexDOI.group(1)
+        dois = re.findall(doi_regexp[version],text,re.I)
+        return dois
     except:
         pass
     return None
 
-def find_identifier_in_google_search(query,func_validate,numb_results=10):
+def find_identifier_in_google_search(query,func_validate,numb_results):
     headers = {"User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36"}
 
     MaxLengthDisplay = 100
@@ -181,7 +235,7 @@ def find_identifier_in_google_search(query,func_validate,numb_results=10):
     else:
         query_to_display = query
     logger.info(f"Performing google search with key \"" + query_to_display + "\"")
-    
+    logger.info(f"and looking at the first {numb_results} results...")
     i=1
     try:
         for url in search(query, stop=numb_results):
@@ -226,23 +280,24 @@ def find_identifier_in_text(texts,func_validate):
     if not isinstance(texts,list): texts = [texts]
     
     for text in texts:
-        #logger.info(text)
-        #logger.info(type(text))
+
         if isinstance(text, bytes):
             text = text.decode()
         #First we look for DOI
         for v in range(len(doi_regexp)):
-            identifier = extract_doi_from_text(text,version=v)
-            validation = func_validate(identifier,'doi')
-            if validation: 
-                return identifier, 'DOI', validation
+            identifiers = extract_doi_from_text(text,version=v)
+            for identifier in identifiers:
+                validation = func_validate(identifier,'doi')
+                if validation: 
+                    return identifier, 'DOI', validation
             
         #Then we look for an Arxiv ID
         for v in range(len(arxiv_regexp)):
-            identifier = extract_arxivID_from_text(text,version=v)
-            validation = func_validate(identifier,'arxiv')
-            if validation:
-                return identifier,'arxiv ID', validation
+            identifiers = extract_arxivID_from_text(text,version=v)
+            for identifier in identifiers:
+                validation = func_validate(identifier,'arxiv')
+                if validation:
+                    return identifier,'arxiv ID', validation
     return None, None, None
 
 
@@ -267,10 +322,16 @@ def get_pdf_info(path):
         return None
     try:
         pdf = PdfFileReader(path,strict=False)
-    except:
+    except Exception as e:
         logger.error("It was not possible to open the file with PyPDF2. Is this a valid pdf file?")
+        logger.error(f"{e}")
         return None
-    info = pdf.getDocumentInfo()
+    try:
+        info = pdf.getDocumentInfo()
+    except Exception as e:
+        logger.error(f"An error occurred when retrieving the pdf info with PyPDF2: {e}")
+        return None
+
     file.close()
     return info
     
@@ -296,6 +357,8 @@ def find_possible_titles(path):
         title = pdftitle.get_title_from_file(path)
     except:
         title = ''
+    if not(isinstance(title, str)):
+        return
     if len(title.strip())>12:#This is to check that the title found is neither empty nor just few characters
         titles.append(title)  
         
@@ -305,7 +368,7 @@ def find_possible_titles(path):
     
     for key, value in info.items():
         if 'title' in key.lower():
-            if isinstance(value,str) and len(value.strip())>12: #This is to check that the title found is neither empty nor just few characters
+            if isinstance(value,str) and len(value.strip())>12 and len(value.split())>3: #This is to check that the title found is neither empty nor just few characters or few words
                 titles.append(value)         
     # (3)
     title = os.path.basename(path)
@@ -338,13 +401,19 @@ def get_pdf_text(path,reader):
             except:
                 logger.error("The input argument 'pdf' must be a valid path to a pdf file.")
                 return None
-            number_of_pages = pdf.getNumPages()
+            try:
+                number_of_pages = pdf.getNumPages()
+            except Exception as e:
+                logger.error(f"An error occurred when retrieving the number of pages in the pdf with PyPDF2.")
+                logger.error("Error from PyPDF2: " + str(e))
+                return None
+            
             for i in range(number_of_pages):
                 try:
                     text.append( (pdf.getPage(i)).extractText())
                 except Exception as e:
-                    logger.error("Error from PyPDF2: " + str(e))
                     logger.error("An error occured while loading the document text with PyPDF2. The pdf version might be not supported.")
+                    logger.error("Error from PyPDF2: " + str(e))
                     break 
     if reader == 'textract':
         try:
@@ -354,16 +423,16 @@ def get_pdf_text(path,reader):
             try:
                 text = [textract.process(path)]
             except Exception as e:   
-                logger.error("Error from textract: " + str(e))
                 logger.error("An error occured while loading the document text with textract. The pdf version might be not supported.")
+                logger.error("Error from textract: " + str(e))
     return text
 
 def add_found_identifier_to_metadata(target,identifier):
-    """Given a pdf file or a folder identified by the input variable target, it adds a metadata with name '/identifier'
-    and containing the content of the input variable identifier to either the pdf files (if target is the path of a single file) or
-    to all the pdf files in a folder (if target is the path of a folder). This can be useful to make sure that the next time
+    """Given a pdf file or a folder identified by the input variable target, it adds a metadata with label '/identifier'
+    and containing the content of the input variable identifier to all pdf files specified by target (either a single file or
+    all the pdf files in a folder). This can be useful to make sure that the next time
     this same pdf is analysed, the identifier is found more easily.
-    It can also be useful when one want to reset to to '' the '/identifier' of all pdf files in a certain folder.
+    It can also be useful when one want to reset to '' the '/identifier' of all pdf files in a certain folder.
 
     Parameters
     ----------
@@ -378,14 +447,14 @@ def add_found_identifier_to_metadata(target,identifier):
     list_files = []
     if  os.path.isdir(target): #if target is a folder, we populate the list list_files with all the pdf files contained in this folder
         logger.info(f"Looking for pdf files in the folder {target}...")
-        pdf_files = [f for f in os.listdir(target) if f.endswith('.pdf')]
+        pdf_files = [f for f in os.listdir(target) if (f.lower()).endswith('.pdf')]
         numb_files = len(pdf_files)
         if len(pdf_files) == 0:
             logger.error("No pdf files found in this folder.")
             return None
         logger.info(f"Found {numb_files} pdf files.")
-        if not(target.endswith(config.separator)): #Make sure the path ends with "\" or "/" (according to the OS)
-            target = target + config.separator
+        if not(target.endswith(config.get('separator'))): #Make sure the path ends with "\" or "/" (according to the OS)
+            target = target + config.get('separator')
         for f in pdf_files:
             list_files.append(target + f)
     else:
@@ -396,21 +465,23 @@ def add_found_identifier_to_metadata(target,identifier):
         try:
             file = open(f, 'rb') 
         except (FileNotFoundError, IOError):
-            logger.error("File not found.")
-            return False
+            msg = "File not found."
+            logger.error(msg)
+            return False, msg
         try:
             pdf = PdfFileReader(f,strict=False)
         except:
-            logger.error("It was not possible to open the file with PyPDF2. Is this a valid pdf file?")
-            return False
+            msg = "It was not possible to open the file with PyPDF2. Is this a valid pdf file?"
+            logger.error(msg)
+            return False, msg
         try:
             writer = PdfFileWriter()
             writer.appendPagesFromReader(pdf)
             metadata = pdf.getDocumentInfo()
             try:
                 writer.addMetadata(metadata)    #This instruction might generate an error if the pre-existing metadata are weird and are not
-            except:                             #correctly seen as strings (happens with old files). Therefore we use the try/except
-                pass                           #to ignore this possible problem
+            except:                             #correctly seen as strings (it happens with old files). Therefore we use the try/except
+                pass                            #to ignore this possible problem
             key = '/identifier'
             writer.addMetadata({
                 key: identifier
@@ -422,8 +493,9 @@ def add_found_identifier_to_metadata(target,identifier):
             logger.info(f"The identifier \'{identifier}\' was added succesfully to the metadata of the file \'{f}\' with key \'{key}\'...")
         except Exception as e:
             logger.error("Error from PyPDF2: " + str(e))
-            logger.error(f"An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{f}\'...")
-            return False
+            msg = f"An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{f}\'. Maybe the file is open elsewhere?"
+            logger.error(msg)
+            return False, msg
 
 
 ######## End first part ######## 
@@ -435,8 +507,8 @@ def add_found_identifier_to_metadata(target,identifier):
 ######## Beginning second part, high-level functions ######## 
 
 '''
-The following functions are high-level identifier finders, i.e. functions that can be called directly by the user or by the main
-script pdf2doi.py. The function find_identifier acts as a wrapper for all the other high-level functions: 
+The following functions are high-level identifier finders, i.e. functions that can be called directly by the user or by the function pdf2doi
+contained in main.py. The function find_identifier acts as a wrapper for all the other high-level functions: 
 based on the value of the input argument method, it call the corresponding function. 
 The correspondence between the values of the string method and the function to call is defined in the dictionary finder_methods 
 (see bottom of this module).
@@ -444,12 +516,12 @@ The correspondence between the values of the string method and the function to c
 
 
 
-def find_identifier(path,method,func_validate=validate,**kwargs):
+def find_identifier(path, method, func_validate=validate,**kwargs):
     """ Tries to find an identifier (e.g. DOI, arxiv ID,...) for the pdf file identified by the input
     argument 'path', by using the method specified by input argument 'method'. Any found identifier is validated
     by using the function func_validate. If a valid identifier is found with any method different from
     "document_infos" (i.e. by looking into the file metadata) the identifier is also added to the file metadata
-    with key "/identifier" (unless config.save_identifier_metadata is set to False)
+    with key "/identifier" (unless config.get('save_identifier_metadata') is set to False)
 
     Parameters
     ----------
@@ -471,8 +543,8 @@ def find_identifier(path,method,func_validate=validate,**kwargs):
     result : dictionary
         result['identifier'] = DOI or other identifier (or None if nothing is found)
         result['identifier_type'] = string specifying the type of identifier (e.g. 'doi' or 'arxiv')
-        result['validation_info'] = Additional info on the paper. If config.check_online_to_validate = True, then result['validation_info']
-                                    will typically contain a bibtex entry for this paper. Otherwise it will just contain True                         
+        result['validation_info'] = Additional info on the paper. If config.get('webvalidation') = True, then result['validation_info']
+                                    will typically contain a raw bibtex data for this paper. Otherwise it will just contain True                         
         result['path'] = path of the pdf file
         result['method'] = method used to find the identifier
 
@@ -482,29 +554,27 @@ def find_identifier(path,method,func_validate=validate,**kwargs):
     if not callable(func_validate):  func_validate = lambda x : x
 
     identifier, desc, info = finder_methods[method](path,func_validate,**kwargs)
-
-    if config.save_identifier_metadata==True:
+    if (config.get('save_identifier_metadata'))==True:
         if identifier and not(method=="document_infos"):
             add_found_identifier_to_metadata(path,identifier)
     
-    result = {'identifier':identifier,'identifier_type':desc,'validation_info':info,
+    result = {'identifier':identifier,'identifier_type':desc,
               'path':path, 'method':method}
+    result['validation_info'] = info
     return result
 
-def find_identifier_by_googling_title(path, func_validate, numb_results=config.numb_results_google_search):
+def find_identifier_by_googling_title(path, func_validate):
     titles = find_possible_titles(path)
-
     if titles:
-        if config.websearch==False:
-            logging.warning("Possible titles of the paper were found, but the web-search method is currently disabled by the user. Enable it in order to perform a qoogle query.")
+        if config.get('websearch')==False:
+            logging.info("NOTE: Possible titles of the paper were found, but the web-search method is currently disabled by the user. Enable it in order to perform a qoogle query.")
             return None, None, None
         else:
             logger.info(f"Found {len(titles)} possible title(s).")
             titles.sort(key=len, reverse=True)
-            for title in titles:
-                logger.info(f"Doing a google search for \"{title}\",")
-                logger.info(f"looking at the first {config.numb_results_google_search} results...")
-                identifier,desc,info = find_identifier_in_google_search(title,func_validate,numb_results)
+            for index_title,title in enumerate(titles):
+                logger.info(f"Trying possible title #{index_title+1}")
+                identifier,desc,info = find_identifier_in_google_search(title,func_validate,numb_results=config.get('numb_results_google_search'))
                 if identifier:
                     logger.info(f"A valid {desc} was found with this google search.")
                     return identifier,desc,info
@@ -514,10 +584,10 @@ def find_identifier_by_googling_title(path, func_validate, numb_results=config.n
         logger.error("It was not possible to find a title for this file.")
         return None, None, None
     
-def find_identifier_by_googling_first_N_characters_in_pdf(path, func_validate, numb_results=config.numb_results_google_search, numb_characters=config.N_characters_in_pdf):
+def find_identifier_by_googling_first_N_characters_in_pdf(path, func_validate, numb_results=config.get('numb_results_google_search'), numb_characters=config.get('N_characters_in_pdf')):
     
-    if config.websearch==False:
-        logger.warning("Web-search methods are currently disabled by the user. Enable it in order to use this method.")
+    if config.get('websearch')==False:
+        logger.info("NOTE: Web-search methods are currently disabled by the user. Enable it in order to use this method.")
         return None, None, None
 
     for reader in reader_libraries:
@@ -526,10 +596,11 @@ def find_identifier_by_googling_first_N_characters_in_pdf(path, func_validate, n
         if text==[] or text=="":
             logger.error(f"The library {reader} could not extract any text from this file.")
             continue
-        
         if isinstance(text,list):
             text = "".join(text)
-            
+        if not(isinstance(text, str)):
+            logger.error(f"The library {reader} could not extract any text from this file.")
+            continue 
         text = re.sub(r'[^\x00-\x7f]',r' ',text)    #Remove all non-text characters from the string text
         for r in ("\n","\r","\t"):
             text = text.replace(r," ")
@@ -540,7 +611,7 @@ def find_identifier_by_googling_first_N_characters_in_pdf(path, func_validate, n
             
         text = text[0:numb_characters]              #Select the first numb_characters characters
 
-        logger.info(f"Doing a google search, looking at the first {config.numb_results_google_search} results...")
+        logger.info(f"Doing a google search, looking at the first {config.get('numb_results_google_search')} results...")
         identifier,desc,info = find_identifier_in_google_search(text,func_validate,numb_results)
         if identifier:
             logger.info(f"A valid {desc} was found with this google search.")
