@@ -3,25 +3,21 @@ import logging
 from os import path, listdir
 import pdf2doi.finders as finders
 import pdf2doi.config as config
-import pyperclip
+#import easygui Modules that are commented here are imported later only when needed, to improve start up time
+#import pyperclip
 
-def pdf2doi(target, verbose=False, websearch=True, webvalidation=True,
-            save_identifier_metadata = config.save_identifier_metadata,
-            numb_results_google_search = config.numb_results_google_search,
-            filename_identifiers = False, filename_bibtex = False,
-            store_bibtex_clipboard = False, store_identifier_clipboard = False):
+def pdf2doi(target):
     ''' This is the main routine of the library. When the library is used as a command-line tool (via the entry-point "pdf2doi") the input arguments
     are collected, validated and sent to this function (see the function main() below).
-    The function tries to extract the DOI (or other identifiers) of the publication in the pdf file whose path is specified in the input variable target. 
+    The function tries to extract the DOI (or other identifiers) of the publication in the pdf files whose path is specified in the input variable target. 
     If target contains the valid path of a folder, the function tries to extract the DOI/identifer of all pdf files in the folder.
     It returns a dictionary (or a list of dictionaries) containing info(s) about the file(s) examined, or None if an error occurred.
-    If the input variables filename_identifiers and/or filename_bibtex are set to a valid string, all identifiers found and/or bibtex entries for
-    all pdf files are saved in text files (see description of input arguments for details)
+
 
     Example:
         import pdf2doi
         path = r"Path\to\folder"
-        result = pdf2doi.pdf2doi(path, verbose=True)
+        result = pdf2doi.pdf2doi(path)
         print(result[0]['identifier'])          # Print doi/identifier of the first pdf file found in this folder
         print(result[0]['identifier_type'])     # Print the type of identifier found (e.g. 'doi' or 'arxiv')
         print(result[0]['method'])              # Print the method used to find the identifier
@@ -30,35 +26,6 @@ def pdf2doi(target, verbose=False, websearch=True, webvalidation=True,
     ----------
     target : string
         Relative or absolute path of a .pdf file or a directory containing pdf files
-    verbose : boolean, optional
-        Increases the output verbosity. The default is False.
-    websearch : boolean, optional
-        If set false, any method to find an identifier which requires a web search is disabled. The default is True.
-    webvalidation : boolean, optional
-        If set false, validation of identifiers via internet queries (e.g. to dx.doi.org or export.arxiv.org) is disabled. 
-        The default is True.
-    save_identifier_metadata : boolean, optional
-        If set True, when a valid identifier is found with any method different than the metadata lookup, the identifier
-        is also written in the file metadata with key "/identifier" (this will speed up future lookup of thi same file). 
-        If set False, this does not happen. The default is True.
-    numb_results_google_search : integer, optional
-        It sets how many results are considered when performing a google search. The default is config.numb_results_google_search.
-    filename_identifiers : string or boolean, optional
-        If set equal to a string, all identifiers found in the directory specified by target are saved into a text file 
-        inside the same directory and with a name specified by filename_identifiers. 
-        The default is False.  It is ignored if the input parameter target is a file.
-    filename_bibtex : string or boolean, optional
-        If set equal to a string, all bibtex entries obtained in the validation process for all pdf files found in the 
-        directory specified by target are saved into a file inside the same directory and with a name specified by filename_bibtex. 
-        The default is False. It is ignored if the input parameter target is a file.
-    store_bibtex_clipboard : boolean, optional
-        If set true, the bibtex entries of all pdf files (or a for a single pdf file if target is a file) are
-        stored in the system clipboard. The default is False. 
-    store_identifier_clipboard : boolean, optional
-        If set true, the identifier of all pdf files (or a for a single pdf file if target is a file) are
-        stored in the system clipboard. The default is False. 
-        If both store_bibtex_clipboard and store_identifier_clipboard are set to true, the bibtex entries have 
-        priority.
 
     Returns
     -------
@@ -68,30 +35,19 @@ def pdf2doi(target, verbose=False, websearch=True, webvalidation=True,
         
         result['identifier'] = DOI or other identifier (or None if nothing is found)
         result['identifier_type'] = string specifying the type of identifier (e.g. 'doi' or 'arxiv')
-        result['validation_info'] = Additional info on the paper. If config.check_online_to_validate = True, then result['validation_info']
-                                    will typically contain a bibtex entry for this paper. Otherwise it will just contain True 
-        result['bibtex_data'] = dictionary containing all available bibtex info of this publication. E.g., result['bibtex_info']['author'], result['bibtex_info']['title'], etc.
+        result['validation_info'] = Additional info on the paper. If config.get('webvalidation') = True, then result['validation_info']
+                                    will typically contain raw bibtex data for this paper. Otherwise it will just contain True 
         result['path'] = path of the pdf file
         result['method'] = method used to find the identifier
 
     ''' 
-    config.check_online_to_validate = webvalidation
-    config.websearch = websearch
-    config.save_identifier_metadata = save_identifier_metadata
-    if numb_results_google_search:
-        config.numb_results_google_search = numb_results_google_search
-
-    # Setup logging
-    if verbose: loglevel = logging.INFO
-    else: loglevel = logging.CRITICAL
 
     logger = logging.getLogger("pdf2doi")
-    logger.setLevel(level=loglevel)
 
     #Check if path is valid
     if not(path.exists(target)):
         logger.error(f"{target} is not a valid path to a file or a directory.")
-        return
+        return None
       
     #Check if target is a directory
     #If yes, we look for all the .pdf files inside it, and for each of them
@@ -106,37 +62,20 @@ def pdf2doi(target, verbose=False, websearch=True, webvalidation=True,
             return None
         
         logger.info(f"Found {numb_files} pdf files.")
-        if not(target.endswith(config.separator)): #Make sure the path ends with "\" or "/" (according to the OS)
-            target = target + config.separator
+        if not(target.endswith(config.get('separator'))): #Make sure the path ends with "\" or "/" (according to the OS)
+            target = target + config.get('separator')
             
         identifiers_found = [] #For each pdf file in the target folder we will store a dictionary inside this list
         for f in pdf_files:
             logger.info("................") 
             file = target + f
             #For each file we call again this function, but now the input argument target is set to the path of the file
-            result = pdf2doi(   target=file, verbose=verbose, websearch=websearch, webvalidation=webvalidation,
-                                save_identifier_metadata = config.save_identifier_metadata,
-                                numb_results_google_search=numb_results_google_search)
+            result = pdf2doi(target=file)
             logger.info(result['identifier'])
             identifiers_found.append(result)
 
         logger.info("................") 
 
-        #If a string was passed via the argument filename_identifiers or if store_identifier_clipboard = True, 
-        #we save all found identifiers in a either text file with name = filename_identifiers
-        #and/or the system clipboard
-        if isinstance(filename_identifiers,str) or store_identifier_clipboard:
-            path_filename_identifiers = target+filename_identifiers if isinstance(filename_identifiers,str) else False
-            save_identifiers(path_filename_identifiers, identifiers_found, store_identifier_clipboard)
-
-                
-        #If a string was passed via the argument filename_bibtex or if store_bibtex_clipboard = True, and if the online validation was used
-        #we save all the bibtex entries collected during validation in either a file with name = filename_bibtex
-        #and/or the system clipboard
-        if (isinstance(filename_bibtex,str) or store_bibtex_clipboard) and config.check_online_to_validate:
-            path_filename_bibtex = target+filename_bibtex if isinstance(filename_bibtex,str) else False
-            save_bibtex(path_filename_bibtex, identifiers_found, store_bibtex_clipboard)
-            
         return identifiers_found
     
     #If target is not a directory, we check that it is an existing file and that it ends with .pdf
@@ -153,14 +92,6 @@ def pdf2doi(target, verbose=False, websearch=True, webvalidation=True,
         if result['identifier'] == None:
             logger.error("It was not possible to find a valid identifier for this file.")
 
-        #The next 4 lines of code need to be improved in next version. Right now, if I am scanning a folder, it will first copy
-        #the details of each file separately into the clipboard, and then in the end it will copy the whole list to the clipboard
-        if store_bibtex_clipboard and config.check_online_to_validate:
-            save_bibtex(False, [result], store_bibtex_clipboard)
-        if  store_identifier_clipboard:
-            save_identifiers(False, [result], store_identifier_clipboard)
-
-                
         return result #This will be a dictionary with all entries as None
 
 def pdf2doi_singlefile(filename):
@@ -177,9 +108,8 @@ def pdf2doi_singlefile(filename):
         
         result['identifier'] = DOI or other identifier (or None if nothing is found)
         result['identifier_type'] = string specifying the type of identifier (e.g. 'doi' or 'arxiv')
-        result['validation_info'] = Additional info on the paper. If config.check_online_to_validate = True, then result['validation_info']
-                                    will typically contain a bibtex entry for this paper. Otherwise it will just contain True 
-        result['bibtex_data'] = dictionary containing all available bibtex info of this publication. E.g., result['bibtex_info']['author'], result['bibtex_info']['title'], etc.
+        result['validation_info'] = Additional info on the paper. If config.get('webvalidation') = True, then result['validation_info']
+                                    will typically contain raw bibtex data for this paper. Otherwise it will just contain True 
         result['path'] = path of the pdf file
         result['method'] = method used to find the identifier
 
@@ -208,7 +138,6 @@ def pdf2doi_singlefile(filename):
     if result['identifier']:
         return result 
     
-        
     #Fourth method: We look for possible titles of the paper, do a google search with them, 
     # open the first results and look for identifiers in the plain text of the searcg results.
     logger.info(f"Method #4: Looking for possible publication titles...")
@@ -216,14 +145,14 @@ def pdf2doi_singlefile(filename):
     if result['identifier']:
         return result
         
-    #Fifth method: We extract the first N characters from the file (where N is set by config.N_characters_in_pdf) and we use it as 
+    #Fifth method: We extract the first N characters from the file (where N is set by config.get('N_characters_in_pdf')) and we use it as 
     # a query for a google seaerch. We open the first results and look for identifiers in the plain text of the searcg results.
-    logger.info(f"Method #5: Trying to do a google search with the first {config.N_characters_in_pdf} characters of this pdf file...")
+    logger.info(f"Method #5: Trying to do a google search with the first {config.get('N_characters_in_pdf')} characters of this pdf file...")
     result =  finders.find_identifier(filename,method="first_N_characters_google")
     if result['identifier']:
         return result
 
-    return result   #If it gets to this line, then no valid identifier was found and
+    return result   #If the execution gets to this line, then no valid identifier was found and
                     #result is a dictionary with result['identifier'] = None 
 
 
@@ -245,68 +174,38 @@ def save_identifiers(filename_identifiers, results, clipboard = False):
     None.
     '''
     logger = logging.getLogger("pdf2doi")
-    if filename_identifiers:
+
+    #If a string was passed via the args.filename_identifiers, we create the full path of the file where identifiers will be saved
+    if isinstance(filename_identifiers,str):
+        path_filename_identifiers = path.dirname(results[0]['path']) + config.get('separator') + filename_identifiers
         try:
             text = ''
             for result in results:
                 if result['validation_info']:
-                    text = text + '{:<15s} {:<40s} {:<10s}\n'.format(result['identifier_type'], result['identifier'],result['path']) 
-            with open(filename_identifiers, "w", encoding="utf-8") as text_file:
+                    text = text + '{:<15s} {:<40s} {:<10s}\n'.format(result['identifier_type'], result['identifier'],result['path'])
+                else:
+                    text = text + '{:<15s} {:<40s} {:<10s}\n'.format('n.a.', 'n.a.',result['path']) 
+            with open(path_filename_identifiers, "w", encoding="utf-8") as text_file:
                 text_file.write(text) 
             logger.info(f'All found identifiers were saved in the file {filename_identifiers}')
         except Exception as e:
             logger.error(e)
             logger.error(f'A problem occurred when trying to write into the file {filename_identifiers}')
+
+    #If clipboard is set to true, we copy all identifiers into the clipboard
     if clipboard:
+        import pyperclip
         try:
             text = ''
             for result in results:
                 if result['validation_info']:
                     text = text + result['identifier'] + '\n' 
             pyperclip.copy(text)
-            logger.info(f'All available identifiers have been stored in the system clipboard')
+            logger.info(f'All found identifiers have been stored in the system clipboard')
         except Exception as e:
             logger.error(e)
             logger.error(f'A problem occurred when trying to write into the system clipboard')
 
-def save_bibtex(filename_bibtex, results, clipboard = False):
-    '''Write all available bibtex entries from results contained in the input list 'results' into a text file with a path specified by filename_bibtex 
-        (if filename_bibtex is a valid string) and/or into the clipboard (if clipboard = True).
-    
-    Parameters
-    ----------
-    filename_bibtex : string
-        Absolute path of the target file. If equal to '' or False, nothing is stored on file.
-    results : list of dictionaries
-        Each element of the list 'results' describes a .pdf file, and contains the pdf identifier and other infos.
-    clipboard : boolean, optional
-        If set to True, the bibtex entries are stored in the clipboard. Default is False.
-        
-    Returns
-    -------
-    None.
-
-    '''
-    logger = logging.getLogger("pdf2doi")
-    text = ''
-    for result in results:
-        if isinstance(result['validation_info'],str):
-            text = text + result['validation_info'] + "\n\n"
-    if filename_bibtex:
-        try:
-            with open(filename_bibtex, "w", encoding="utf-8") as text_file:
-                text_file.write(text) 
-            logger.info(f'All available bibtex entries have been stored in the file {filename_bibtex}')
-        except Exception as e:
-            logger.error(e)
-            logger.error(f'A problem occurred when trying to write into the file {filename_bibtex}')
-    if clipboard:
-        try:
-            pyperclip.copy(text)
-            logger.info(f'All available bibtex entries have been stored in the system clipboard')
-        except Exception as e:
-            logger.error(e)
-            logger.error(f'A problem occurred when trying to write into the system clipboard')
     
 def main():
     parser = argparse.ArgumentParser( 
@@ -317,9 +216,9 @@ def main():
                         help = "Relative path of the target pdf file or of the targe folder.",
                         metavar = "path",
                         nargs = '*')
-    parser.add_argument("-nv",
-                        "--no_verbose",
-                        help="Decrease verbosity.",
+    parser.add_argument("-v",
+                        "--verbose",
+                        help="Increase verbosity. By default (i.e. when not using -v), only a table with the found identifiers will be printed as output.",
                         action="store_true")
     parser.add_argument("-nws",
                         "--no_web_search",
@@ -331,7 +230,7 @@ def main():
                         action="store_true")
     parser.add_argument("-nostore",
                         "--no_store_identifier_metadata",
-                        help="By default, anytime an identifier is found it is added to the metadata of the pdf file (if not present yet). By setting this parameter, the identifier is not stored in the file metadata.",
+                        help="By default, anytime an identifier is found it is added to the metadata of the pdf file (if not present yet). By using this additional option, the identifier is not stored in the file metadata.",
                         action="store_true")
     parser.add_argument('-id', 
                         help=f"Stores the string IDENTIFIER in the metadata of the target pdf file, with key \'/identifier\'. Note: when this argument is passed, all other arguments (except for the path to the pdf file)" +
@@ -344,24 +243,15 @@ def main():
                                         #.pdf file in Windows
                         help=argparse.SUPPRESS,
                         action="store_true")
-    parser.add_argument('-google_results', 
-                        help=f"Set how many results should be considered when doing a google search for the DOI (default={str(config.numb_results_google_search)}).",
+    parser.add_argument('-google', 
+                        help=f"Set how many results should be considered when doing a google search for the DOI (default={str(config.get('numb_results_google_search'))}).",
                         action="store", dest="google_results", type=int)
     parser.add_argument("-s",
                         "--save_identifiers_file",
                         dest="filename_identifiers",
                         help="Save all the identifiers found in the target folder in a text file inside the same folder with name specified by FILENAME_IDENTIFIERS. This option is only available when a folder is targeted.",
                         action="store")
-    parser.add_argument("-b",
-                        "--make_bibtex_file",
-                        dest="filename_bibtex",
-                        help="Create a text file inside the target directory with name given by FILENAME_BIBTEX containing the bibtex entry of each pdf file in the target folder (if a valid identifier was found). This option is only available when a folder is targeted, and when the web validation is allowed.",
-                        action="store")
-    parser.add_argument("-bclip",
-                        "--save_bibtex_clipboard",
-                        action="store_true",
-                        help="Store all found bibtex entries into the clipboard.")
-    parser.add_argument("-doiclip",
+    parser.add_argument("-clip",
                         "--save_doi_clipboard",
                         action="store_true",
                         help="Store all found DOI/identifiers into the clipboard.")
@@ -377,12 +267,17 @@ def main():
 
     args = parser.parse_args()
 
+    # Setup logging
+    config.set('verbose',args.verbose) #store the desired verbose level in the global config of pdf2doi. This will automatically update the logger verbosity
+
     #If the command -install--right--click was specified, it sets the right keys in the system registry
     if args.install_right_click:
+        config.set('verbose',True)
         import pdf2doi.utils_registry as utils_registry
         utils_registry.install_right_click()
         return
     if args.uninstall_right_click:
+        config.set('verbose',True)
         import pdf2doi.utils_registry as utils_registry
         utils_registry.uninstall_right_click()
         return
@@ -409,20 +304,20 @@ def main():
     #Nothing else will be done
     if args.identifier or args.identifier=="":
         if isinstance(args.identifier,str):
-            finders.add_found_identifier_to_metadata(target,args.identifier)
+            result = finders.add_found_identifier_to_metadata(target,args.identifier)
+            if args.id_input_box and len(result)>1:
+                if result[0]==False:
+                    easygui.msgbox(result[1])
         return
 
-    results = pdf2doi(target=target,
-                  verbose=not(args.no_verbose),
-                  websearch=not(args.no_web_search),
-                  webvalidation=not(args.no_web_validation),
-                  save_identifier_metadata = not(args.no_store_identifier_metadata),
-                  numb_results_google_search=args.google_results,
-                  filename_identifiers = args.filename_identifiers,
-                  filename_bibtex = args.filename_bibtex,
-                  store_bibtex_clipboard = args.save_bibtex_clipboard, 
-                  store_identifier_clipboard = args.save_doi_clipboard
-                  )
+    config.set('websearch',not(args.no_web_search))
+    config.set('webvalidation',not(args.no_web_validation))
+    config.set('save_identifier_metadata',not(args.no_store_identifier_metadata))
+    
+    if args.google_results:
+        config.set('numb_results_google_search',args.google_results)
+    results = pdf2doi(target=target)
+
     if not results:
         return
     if not isinstance(results,list):
@@ -430,6 +325,13 @@ def main():
     for result in results:
         if result['identifier']:
             print('{:<15s} {:<40s} {:<10s}\n'.format(result['identifier_type'], result['identifier'],result['path']) ) 
+        else:
+            print('{:<15s} {:<40s} {:<10s}\n'.format('n.a.', 'n.a.',result['path']) ) 
+
+
+    # We call the function save_identifiers. If args.filename_identifiers is a valid string, it will save all found identifiers in a text file with that name.
+    # If args.save_doi_clipboard is true, it will copy all identifiers into the clipboard
+    save_identifiers(args.filename_identifiers, results, args.save_doi_clipboard)  
 
     return
 
